function  [Pout_ASE_dBm,z] = Raman_ASE_noise(c,a,NA,freq_ref_raman,...
    param,z,P_ASE_W_start,freq,...
    C_R,f_C_R,alpha_0_vec_freq_dep,k_Boltz,h_Planck,T_room,delta_f,delta_z)
% this function is for calculating the ASE noise from Raman with the power
% profile found in a fiber

global  C F alpha N P_col deltaf_Hz freq_Hz h_P k_B T_R dz
 
deltaf_Hz = delta_f;
dz = delta_z;
h_P = h_Planck;
alpha = alpha_0_vec_freq_dep';
k_B = k_Boltz;
T_R = T_room;
P_lin_W = P_ASE_W_start;
N = length(P_lin_W);
P_col=P_lin_W';
freq_Hz = freq;
param = squeeze(param);
F1 = repmat(freq_Hz,N,1);
F2 = repmat(freq_Hz',1,N);

C = (F1>=F2).*C_R_R(c,a,NA,F1-F2,F1,freq_ref_raman,C_R,f_C_R)-...
    (F2>F1).*(F2./F1).*C_R_R(c,a,NA,F2-F1,F2,freq_ref_raman,C_R,f_C_R);

F = (F1>F2).*(photon_generate(abs(F1-F2)) + 1) + ...
    (F1<F2).*(-photon_generate(abs(F1-F2)));
F(isnan(F)) = 0;

% load('num_power_evolution.mat','z','Pout_dBm')
% param = 10.^(Pout_dBm/10) * 1e-3;
options = odeset('AbsTol',1e-6,'RelTol',1e-6);%,
[z,Pout_lin_W] = ode45(@(z,y)bvp(z,y,param),z,P_col,options);
Pout_ASE_dBm = 10*log10(Pout_lin_W * 1e3);
end


% % distribute Raman Gain on each pair of channels
% function crr = C_R_R(f,C_R,f_C_R)
% 
% global N
% M=length(f_C_R);
% for xx=1:N
%     for yy=1:N
%         AA=(find(f_C_R<=f(xx,yy)));
%         if length(AA)==0
%             crr(xx,yy)=C_R(1);
%         elseif length(AA)==M
%             crr(xx,yy)=C_R(M);
%         else
%             n1=max(AA);
%             n2=n1+1;
%             mm=(C_R(n2)-C_R(n1))/(f_C_R(n2)-f_C_R(n1));
%             b=C_R(n2)-mm*f_C_R(n2);
%             crr(xx,yy)=mm*f(xx,yy)+b;
%         end
%     end
% end
% end

% number of phontons generated between each pair of channels
function Nphon = photon_generate(f)
global h_P k_B T_R
Nphon = 1./(exp(h_P.*abs(f)/(k_B*T_R))-1);
end


% ASE power evolution function
function dydz = bvp(z,y,param)
global  C F alpha h_P freq_Hz deltaf_Hz dz

dydz = -(alpha).*(y)+ C*param(floor(z/dz)+1,:)'.*y + 2*h_P*freq_Hz'.*deltaf_Hz'.*(C.*F*param(floor(z/dz)+1,:)');

end
