function  [Pout_dBm,z] = Raman_forward_backward(c,a,NA,...
                        freq_ref_raman,P_dBm_boundary,...
                        freq_center,propagating_direction,C_R,...
                        f_C_R,alpha0,deltaz,Ls_km)

global  C alpha N direc_v P_col Ls_m zmesh
direc_v = propagating_direction';
alpha = alpha0';
P_lin_W=10.^(P_dBm_boundary * 0.1) * 1e-3;
N = length(P_lin_W);
P_col = P_lin_W';
freq = freq_center;
F1 = repmat(freq,N,1);
F2 = repmat(freq',1,N);
C = (F1>=F2).*C_R_R(c,a,NA,F1-F2,F1,freq_ref_raman,C_R,f_C_R)-...
    (F2>F1).*(F2./F1).*C_R_R(c,a,NA,F2-F1,F2,freq_ref_raman,C_R,f_C_R);

Ls_m = Ls_km*1000;
z1 = 0:deltaz:Ls_m;
if (sum(direc_v<0))==0
    options = odeset('AbsTol',1e-6,'RelTol',1e-6);%,
    [z,Pout_lin_W] = ode45(@bvpfcn_fwd,z1,P_col',options);
    Pout_lin_W(Pout_lin_W<=0)=1e-30;
    Pout_dBm=(10*log10(1000*Pout_lin_W));
else
%     load('num_power_evolution_nonuniform.mat','z')
%     zmesh = sort([z,(z(1:end-1) + z(2:end))/2]);
    zmesh = 0:deltaz:Ls_m;
    options = odeset('AbsTol',1e-6,'RelTol',1e-8);%,
    P_init_fwd = P_col.*(direc_v>0);
    [~,Pout_lin_W1] = ode45(@bvpfcn_fwd,zmesh,P_init_fwd,options);
    Pout_lin_W1(Pout_lin_W1<=1e-30) = 1e-30;
    P_init_bwd = P_col.*(direc_v<0);
    [~,Pout_lin_W2] = ode45(@bvpfcn_fwd,zmesh,P_init_bwd,options);
    Pout_lin_W2(Pout_lin_W2<=1e-30) = 1e-30;
    Pout_lin_W3 = flipud(Pout_lin_W2);
    
    power_guess = Pout_lin_W1 .* repmat((direc_v'>0), length(zmesh),1) +...
        Pout_lin_W3 .* repmat((direc_v'<0), length(zmesh),1);
    
    solinit = bvpinit(zmesh, @guess); 
    solinit.y = power_guess';
    options = bvpset('RelTol',1e-6,'AbsTol',1e-6,'stats','on');
    sol = bvp4c(@bvpfcn_bwd, @bcfcn, solinit,options);
    z = sol.x;
    Pout_dBm=(10*log10(1000*sol.y))';
end
end

% function crr = C_R_R(c,a,NA,f,f_p,f_p_r,C_R,f_C_R)
% global N
% M = length(f_C_R);
% for xx = 1:N
%     for yy = 1:N
%         AA = find(f_C_R<=f(xx,yy));
%         if isempty(AA)
%             crr(xx,yy) = C_R(1);
%         elseif length(AA) == M
%             crr(xx,yy) = C_R(M);
%         else
%             n1 = max(AA);
%             n2 = n1+1;
%             mm = (C_R(n2)-C_R(n1))/(f_C_R(n2)-f_C_R(n1));
%             b = C_R(n2)-mm*f_C_R(n2);
%             crr(xx,yy) = mm*f(xx,yy)+b;
%             
%         end
%     end
% end
% crr = crr.*((f_p-f)./(f_p_r-f)).*((A_eff_2_D(f_p_r,c,NA,a)+ ...
%     A_eff_2_D(f_p_r-f,c,NA,a))./(A_eff_2_D(f_p,c,NA,a)+...
%     A_eff_2_D(f_p-f,c,NA,a)));
% 
% crr(isinf(crr) | isnan(crr)) = 0;
% end

function dydz = bvpfcn_fwd(z,y)
global  C  alpha N 
%y=y.*(y>0);
dydz = ((repmat(y,1,N).*C)*y-(alpha).*(y));
end

function dydz = bvpfcn_bwd(z,y)
global  C  alpha N  direc_v
%y=y.*(y>0);
dydz = ((repmat(y,1,N).*C)*y-(alpha).*(y)).* direc_v;
end


function res = bcfcn(ya,yb)
global direc_v P_col
res=(direc_v>0).*(ya-P_col)+(direc_v<0).*(yb-P_col);
end

function g = guess(x)
global  direc_v P_col Ls_m alpha
g=(direc_v>0).*P_col.*exp(-alpha*x)+(direc_v<0).*P_col.*exp(+alpha*(x-Ls_m));
end
