function [P,iter_number,correction_factor_lower,oscillation_flag]=correction_factor_power_profile_calculator(P,c,a,NA,freq_ref_raman,P_dBm_boundary,freq_center, ...
                                                                propagating_direction,C_R,f_C_R,alpha_lin,Zk,L_s, ...
                                                                correction_factor_lower,correction_factor_higher, ...
                                                                T_trig,plotting,stopping_number)
    
    oscillation_flag=0;
    oscillation_check=1;
    peak_mag_thresh=0.3;
    peak_number_thresh=2;

    % no_change=0;
    % no_change_threshold=3;
    change_counter=0;
    last_change=1;
    c_0=1;

    %flags for testing
    
    plotting_signals_all=0;
    plotting_pumps_all=0;
    testing=0;
    pausing=0;
    
    test=0;
    if test
        plotting_signals_all=1;
        plotting_pumps_all=1;
        testing=1
    end

    if pausing 
        plotting_signals_all=1;
        plotting_pumps_all=1;
    end

    Nch_signal=sum(propagating_direction==1);
    Nch_pump=length(propagating_direction)-Nch_signal;
    P_W_boundary=10.^((P_dBm_boundary-30)/10);
    P_pump_mW=P_W_boundary(Nch_signal+1:end)*1e3;
    P_pump_dBm=10*log10(P_pump_mW);

    global N
    N=Nch_signal+Nch_pump;
    freq = freq_center;
    F1 = repmat(freq,N,1);
    F2 = repmat(freq',1,N);
    C = (F1>=F2).*C_R_R(c,a,NA,F1-F2,F1,freq_ref_raman,C_R,f_C_R)-...
    (F2>F1).*(F2./F1).*C_R_R(c,a,NA,F2-F1,F2,freq_ref_raman,C_R,f_C_R);


    err_tol=2e-3;
    k=100; %check every 100 iterations for oscillation
    oscillation_check_values=k:k:stopping_number;
    limitation_counter=0;
    iter_number=0;
    pump_corrections=zeros(1,Nch_pump);

    pump_correction=P_pump_mW*1e-3 - P(Nch_signal+1:end,end)';
    while(any(abs(pump_correction) > err_tol))
        
        if iter_number>= stopping_number
            %disp(["stoped after ", num2str(stopping_number)," iterations"]);
            oscillation_flag=1;
            break;
        end
        iter_number=iter_number + 1;
        if iter_number~=1
            
            for j=1:Nch_pump  % correction loop
                         
                 if pump_correction(j)>0             %under calculated
                      P(Nch_signal+j,:)=P(Nch_signal+j,:)*(1 + correction_factor_lower * pump_correction(j)); % if correction is positive, the calculated pump power at the end of the link is lower than it should be, so we should increase the pump power at the beginning of the link.
                 else
                     limit=-0.5; %Correction limitation:
                    
                      if(correction_factor_higher * pump_correction(j) < limit)
                          P(Nch_signal + j,:)=P(Nch_signal+j,:)*(1+limit);
                          limitation_counter=limitation_counter + 1;
                      else
                          P(Nch_signal + j,:)=P(Nch_signal+j,:)*(1 + correction_factor_higher * pump_correction(j)); % if correction is negative, the caluclated pump power is higher than it should be, to avoid divergence we scale down the whole pump profile 
                      end
                 end
            end
        end
        if plotting==1
            if testing==1
                P_dbm=10*log10(P*1e3);
                figure
                hold on;
                    for j=1:Nch_pump
                        h_line = plot(Zk*1e-3, P_dbm(Nch_signal+j,:));
                         % Get the color of the current line
                        line_color = get(h_line, 'Color');
                        % Plot the marker separately 
                        h_marker = plot(L_s*1e-3, P_pump_dBm(j), '*','color',line_color);
                        title(["after correction iteration ", num2str(iter_number-1)]);
                        
                    end
            end
        end
        a=-propagating_direction'.*(alpha_lin)'.*repmat(Zk,Nch_signal+Nch_pump,1);
        b= propagating_direction'.* C* P * T_trig;
        P=P(:,1).*exp(a + b);
    
    
       
        pump_corrections(iter_number,:)=P_pump_mW*1e-3-P(Nch_signal+1:end,end)';
        %check for oscellation every 100 iterations
        if oscillation_check
            if any(iter_number==oscillation_check_values)
                % if iter_number==600
                %     salatelzen=1;
                % end
                Pump_Error=pump_corrections(last_change:iter_number,1);
                peaks = findpeaks(Pump_Error);
                %peaks=peaks(peaks>peak_mag_thresh*max(peaks));
                peaks=peaks(peaks>peak_mag_thresh*max(Pump_Error));
                
                if length(peaks)> peak_number_thresh
                    correction_factor_lower=correction_factor_lower/((length(peaks)*c_0))%^change_counter
                    last_change=iter_number
                    change_counter=change_counter+1;
                % else
                %     no_change=no_change+1;
                %     if no_change==no_change_threshold
                %      learning_factor_lower=learning_factor_lower*2
                %      last_change=iter_number;
                %     end
                end
            end
        end
    
        pump_correction=P_pump_mW*1e-3-P(Nch_signal+1:end,end)';
        P_dbm=10*log10(P*1e3);
        %plot the resulting power profiles
        if plotting==1
           
            if plotting_signals_all==1
                 if pausing
                    figure(2) 
                 else 
                     figure
                 end
    
                for j=1:5:Nch_signal
                plot(Zk*1e-3,P_dbm(j,:));
                 xlabel("Z (km)");
                ylabel("Pump Power (dBm)");
                title(["Pump Power Profile iteration ", num2str(iter_number)]);
                hold on;
                end
            end
            if plotting_pumps_all==1
                if plotting_signals_all==0 %initialize a figure in the case pumps were to be plotted alone
                    figure
                end
                hold on;
                for j=1:Nch_pump
                    h_line = plot(Zk*1e-3, P_dbm(Nch_signal+j,:));
                     % Get the color of the current line
                    line_color = get(h_line, 'Color');
                    % Plot the marker separately 
                    h_marker = plot(L_s*1e-3, P_pump_dBm(j), '*','color',line_color);
                    
                end
                 if pausing
                    hold off
                    grid on
                    pause
                 end 
    
            end
        end
        
    end
   
    if plotting
        if ~isequal(pump_corrections, zeros(1,Nch_pump))
            figure
            for j=1:Nch_pump
                
                    plot(1:iter_number,pump_corrections(:,j)');
                    xlabel("iteration number")
                    ylabel("P_{ideal} - P_{calc} (W) ")
                    title("Pump Input Power Error");
                    hold on;

            end
        end
    end

end