function [P,Zk,iter_number,divergence_flag,oscillation_flag]=signal_scale_down_power_profile_calculator(c,a_fiber,NA,freq_ref_raman,P_dBm_boundary,grid_center, ...
                                                                propagating_direction,C_R,f_C_R,alpha_lin,deltaZ,L_s, ...
                                                                plotting)
    
    %output flags
    divergence_flag=0;
    stopping_number=3000;

    %parameters
    factor_signal=4;
    k=1;
    factor_pumps=[1 5 10 15 25 50];
    factor_pump=factor_pumps(k);
    dBm_step=2; %changed from 0.25 
    additional_iterations=20; %step pump

    correction_factor_higher=3; %changed from 1
    correction_factor_lower=0.1;


    
    %useful variables
    Nch_signal=sum(propagating_direction==1);
    Nch_pump=length(propagating_direction)-Nch_signal;
   
    P_pump_dBm=P_dBm_boundary(Nch_signal+1:end);
    P_signal_dBm=P_dBm_boundary(1:Nch_signal);
    P_lin_W=10.^(P_dBm_boundary * 0.1) * 1e-3;

    freq_pump_THz=grid_center(Nch_signal+1:end)*1e-12;
    alpha_lin_pump=alpha_lin(Nch_signal+1:end);
    alpha_dB=alpha_lin*(10000*log10(exp(1)));
    alpha_dB_signal=alpha_dB(1:Nch_signal);


    
    %generating Raman coefficients
    global N
    N=Nch_signal+Nch_pump;
    freq = grid_center;
    F1 = repmat(freq,N,1);
    F2 = repmat(freq',1,N);
    C = (F1>=F2).*C_R_R(c,a_fiber,NA,F1-F2,F1,freq_ref_raman,C_R,f_C_R)-...
    (F2>F1).*(F2./F1).*C_R_R(c,a_fiber,NA,F2-F1,F2,freq_ref_raman,C_R,f_C_R);
    
    %% initialisation of the loop
    %getting initial pump profile without depletion
    [P_pump_evol,z] = Raman_forward_backward(c,a_fiber,NA,...
                                        freq_ref_raman,P_pump_dBm,...
                                        freq_pump_THz*1e12,ones(1,Nch_pump),C_R,...
                                        f_C_R,alpha_lin_pump,deltaZ,L_s*1e-3); 
    Zk=z';
    
    P_pump_evol=P_pump_evol';
    deltaZ=diff(Zk);
    deltaZ=[0 deltaZ];
    
    %generating the T_trig matrix
    Nz=length(Zk);
    T_trig=T_trig_creator(Nz,deltaZ);
    P_pump=10.^(P_pump_evol/10)*1e-3;
    NaNflag=1;
    iter_number=0;
    %% Loop that ensures convergence by means of a higher pump scale down factor
    while(NaNflag)
        P=zeros(Nch_signal+ Nch_pump, Nz);
        %0th iteration, pump powers initialised with ISRS and signal power scaled
        %down by factor and affected by attenuation
    
        P_0=10.^(P_signal_dBm/10)*1e-3; %multiply by 1e-3 to go to W
        P_signal=repmat(P_0',1,Nz); %in W
        atten=alpha_dB_signal'.*Zk*1e-3;
        %Signals attenuated and scaled down
        P(1:Nch_signal,:)=10.^((10*log10(P_signal)-propagating_direction(1:Nch_signal)'.*atten)/10)/factor_signal; % in  W
        %pumps scaled down
        P(Nch_signal+1:end,:)= P_pump(:,end:-1:1)/factor_pump; %Add ISRS pumps to the matrix after reinverting
        P_dbm=10*log10(P*1e3);
      
    
        %% loop
        
        err_tol=-1e-3; 
        final_iteration_flag=0;
        counter=0;
     
        P_lin_W_scaled=P_lin_W(Nch_signal+1:end)/factor_pump;
        %calculate the differnce in db 
        dBm_step_pump=10*log10(P_lin_W(end)/P_lin_W_scaled(end))/additional_iterations; %go up in pump power with a fixed number of iterations

        while(any(P_dbm(1:Nch_signal,1)-P_signal_dBm'<err_tol) || final_iteration_flag)
            iter_number=iter_number + 1;
            
            % correction loop
            for j=1:Nch_pump  
                 P(Nch_signal+j,:)=P(Nch_signal+j,:)*P_lin_W_scaled(j)/P(Nch_signal+j,end);
            end
            error=max(10*log10(P(1:Nch_signal,1)*1e3)-P_signal_dBm');

            %% the different phases of the algorithm happen inside this if condition
            if final_iteration_flag
                if factor_pump~=1 && counter<additional_iterations   %pumps are being scaled up    
                        P(Nch_signal+1:end,:)=10.^((10*log10(P(Nch_signal+1:end,:)*1e3)+dBm_step_pump)/10)*1e-3; %increase pump power
                        P_lin_W_scaled=10.^((10*log10(P_lin_W_scaled*1e3)+dBm_step_pump)/10)*1e-3;
                       counter=counter+1;
                %else
                end
                if factor_pump==1 || counter==additional_iterations %correction factor algorithm to ensure low error
                    a=-propagating_direction'.*(alpha_lin)'.*repmat(Zk,Nch_signal+Nch_pump,1);
                    b= propagating_direction'.* C* P * T_trig;
                    P=P(:,1).*exp(a + b);
                    [P,additional_iter_number,correction_factor_lower,oscillation_flag]=correction_factor_power_profile_calculator(P,c,a_fiber,NA,freq_ref_raman,P_dBm_boundary, ...
                        grid_center,propagating_direction,C_R,f_C_R,alpha_lin, ...
                        Zk,L_s,correction_factor_lower, ...
                        correction_factor_higher,T_trig,plotting,stopping_number);
                    iter_number=iter_number + additional_iter_number;
                    break;
                end
            else %increase signal power          
                if abs(error)<dBm_step
                    P(1:Nch_signal,:)=10.^((10*log10(P(1:Nch_signal,:)*1e3)+abs(error))/10)*1e-3; %increase signal power by the increment 
                else
                    P(1:Nch_signal,:)=10.^((10*log10(P(1:Nch_signal,:)*1e3)+dBm_step)/10)*1e-3; %increase signal power by the increment 
                end
                
                P_dbm=10*log10(P*1e3);
                
            end
            %% propagation after profiles have been tampered with
            a=-propagating_direction'.*(alpha_lin)'.*repmat(Zk,Nch_signal+Nch_pump,1);
            b= propagating_direction'.* C* P * T_trig;
            P=P(:,1).*exp(a + b);
            P_dbm=10*log10(P*1e3);
           
            if ~any(P_dbm(1:Nch_signal,1)-P_signal_dBm'<err_tol) % check if signal up scaling is done to move to pump scaling 
                final_iteration_flag=1;
            end
        end
        if ~any(isnan(P))
            NaNflag=0; %Algorithm has converged
        else
            NaNflag=1;
             k=k+1;
            if k>length(factor_pumps)
                divergence_flag=1;
                break; % algorithm has diverged even for high pump scale down
            end
            factor_pump=factor_pumps(k); % algorithm is diverging try a higher pump scale down
        end

    end
    shutup=1;
    if shutup
    if oscillation_flag
        disp(['the algorithm did not converge after ',num2str(stopping_number),' iterations, moving to the reference method']);
    elseif divergence_flag
        disp(['The algorithm Diverged even after a pump scale down factor of', num2str(factor_pump/2),'moving to the reference method']);
    else
        disp(['the algorithm converged to an absolute error of ', num2str(err_tol), ' after ', num2str(iter_number), ' iterations']);
        disp(['the algorithm converged to an absolute error of ', num2str(abs(err_tol)), ' after ', num2str(iter_number-additional_iter_number), '+',num2str(additional_iter_number),' iterations']);
    end
    end
    
end






